@@ -2,23 +2,23 @@
 class unixfile: public file
 {
  public:
-    unixfile(char *name);
+    unixfile(const char *name);
     unixfile();
     ~unixfile();
 
     virtual int setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode, int flags);
     
-    virtual int   open(char *mode);
-    virtual int   open(char *basename, char *mode);
+    virtual int   open(const char *mode);
+    virtual int   open(const char *basename, const char *mode);
     virtual int   close();
     virtual int   read(char *buf,  int len);
     virtual int   write(char *buf, int len);
     virtual int   next();
-    virtual void  perror(char *);
+    virtual void  perror(const char *);
     virtual char *get_basename(int keep_version);
     virtual char *get_printname();
     virtual char *get_printname(char *filename);
-    virtual char *get_format_name();
+    virtual const char *get_format_name();
     virtual int   get_umask();
     virtual int   set_umask(int mask);
     virtual bool  eof();
