@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 2002-2005      P.J. Caulfield          patrick@debian.org
+    (c) 2002-2008      P.J. Caulfield          patrick@debian.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -10,8 +10,7 @@
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-    ******************************************************************************
-    */
+*******************************************************************************/
 
 #include <stdio.h>
 #include <string.h>
@@ -50,6 +49,7 @@ static char prompt_len=0;
 static char line_start_pos=0;
 static char esc_buf[132];
 static int  esc_len=0;
+static int  esc_state=0;
 static int  max_read_len = sizeof(input_buf);
 static int  echo = 1;
 static int  reading = 0;
@@ -70,6 +70,90 @@ int  (*send_input)(char *buf, int len, int term_pos, int flags);
 int  (*send_oob)(char oobchar, int discard);
 void (*rahead_change)(int count);
 
+/* Escape parser. See appx B of cterm.txt.
+   returns 0 if we have terminated the sequenec (or an error)
+   returns 1 if we are stll in an escape sequence */
+
+/* Note that all the numbered rules fall through to the next rule */
+static int parse_escape(char c, int *state)
+{
+	switch (*state)
+	{
+	case 0:
+		if (c == '?' || c == ';')
+		{
+ 			*state = 10;
+			return 1;
+		}
+		if (c == 'O')
+		{
+			*state = 20;
+			return 1;
+		}
+		if (c == 'Y')
+		{
+			*state = 30;
+			return 1;
+		}
+		if (c == '[')
+		{
+			*state = 15;
+			return 1;
+		}
+
+	case 10:
+		if (c >= 32 && c <= 47)
+		{
+			*state = 10;
+			return 1;
+		}
+		if (c >= 48 && c <= 126)
+		{
+			*state = 0;
+			return 0;
+		}
+
+	case 15:
+		if (c >= 48 && c <= 63)
+		{
+			*state = 15;
+			return 1;
+		}
+
+	case 20:
+		if (c >= 32 && c <= 47)
+		{
+			*state = 20;
+			return 1;
+		}
+		if (c >= 64 && c <= 126)
+		{
+			*state = 0;
+			return 0;
+		}
+
+	case 30:
+		if (c >= 32 && c <= 126)
+		{
+			*state = 40;
+			return 1;
+		}
+
+	case 40:
+		if (c >= 32 && c <= 126)
+		{
+			*state = 0;
+			return 0;
+		}
+		break;
+	default:
+		*state = 0;
+		return 0;
+	}
+	return 0;
+}
+
+
 static void send_input_buffer(int flags)
 {
 	char buf[1024];
@@ -80,6 +164,7 @@ static void send_input_buffer(int flags)
 	send_input(buf, input_len, input_len-1, flags);
 	input_len = input_pos = 0;
 	reading = 0;
+	DEBUG_TTY("clearing 'reading' flag\n");
 	echo = 1;
 	interpret_escape = 0;
 }
@@ -87,21 +172,78 @@ static void send_input_buffer(int flags)
 /* Called when select() times out */
 void tty_timeout()
 {
-	if (debug & 4)
-		fprintf(stderr, "TTY: timeout\n");
+	DEBUG_TTY("timeout\n");
 	send_input_buffer(SEND_FLAG_TIMEOUT);
 }
 
 /* Raw write to terminal */
 int tty_write(char *buf, int len)
 {
+	int in_esc = 0; /* escapes can't straddle writes */
+	int esc_state = 0;
+
 	if (discard)
 		return len;
 
+	/* Ignore NULs */
 	if (len == 1 && *buf == 0)
 		return len;
+
+	/* FF is a special case (perhaps!) */
+	if (len == 1 && buf[0] == '\f')
+	{
+		last_char = '\f';
+		return write(termfd, "\033[H\033[2J", 7);
+	}
+
+	if (debug & DEBUG_FLAG_TTY2)
+	{
+		int i;
+		DEBUG_TTY2("Printing %d: ", len);
+		for (i=0; i<len; i++)
+		{
+			if (isgraph(buf[i]))
+			{
+				DEBUGLOG(DEBUG_FLAG_TTY2,  "%d: %c ", i, buf[i]);
+			}
+			else
+			{
+				DEBUGLOG(DEBUG_FLAG_TTY2,  "%d: 0x%02x ", i, (unsigned char)buf[i]);
+			}
+		}
+		DEBUGLOG(DEBUG_FLAG_TTY2, "\n");
+	}
+
 	write(termfd, buf, len);
-	last_char = buf[len-1];
+
+	if (len)
+	{
+		/* if interpret_escape is set then we don't include escape sequences
+		   as possible "last_char"s
+		*/
+		if (!interpret_escape)
+		{
+			int i;
+			for (i=0; i<len; i++)
+			{
+				if (buf[i] == ESC)
+					in_esc = 1;
+				else
+				{
+					if (!in_esc && buf[i])
+						last_char = buf[i];
+					if (in_esc && !parse_escape(buf[i], &esc_state))
+						in_esc = 0;
+				}
+			}
+		}
+		else
+		{
+			last_char = buf[len-1];
+		}
+
+		DEBUG_TTY("Setting last_char to %02x, interpret_escape=%d\n", last_char, interpret_escape);
+	}
 	return len;
 }
 
@@ -114,10 +256,10 @@ void tty_format_cr()
 {
 	char lf = '\n';
 
-	if (debug & 4)
-		fprintf(stderr, "TTY: format_cr, last char was %x\n", last_char);
+	DEBUG_TTY("format_cr, last char was %x\n", last_char);
+
 	if (last_char == '\r')
-		write(termfd, &lf, 1);
+		tty_write(&lf, 1);
 }
 
 void tty_set_noecho()
@@ -142,7 +284,6 @@ void tty_allow_edit(int onoff)
 
 int tty_set_escape_proc(int onoff)
 {
-	int oldval = interpret_escape;
 	interpret_escape = onoff;
 
 	return interpret_escape;
@@ -155,16 +296,14 @@ int  tty_get_input_count(void)
 
 void tty_echo_terminator(int a)
 {
-	if (debug & 4)
-		fprintf(stderr, "TTY: echo terminators = %d\n", a);
+	DEBUG_TTY("echo terminators = %d\n", a);
 
 	echo_terminator = a;
 }
 
 void tty_set_default_terminators()
 {
-	if (debug & 4)
-		fprintf(stderr, "TTY: set default terminators\n");
+	DEBUG_TTY("set default terminators\n");
 
 	/* All control chars except ^R ^U ^W, BS & HT */
 	/* ie 18, 21, 23, 8, 9 */
@@ -179,13 +318,13 @@ void tty_set_default_terminators()
 
 void tty_set_terminators(char *buf, int len)
 {
-	if (debug & 4) {
+	if (debug & DEBUG_FLAG_TTY) {
 		int i;
-		fprintf(stderr, "TTY: set terminators... %d bytes\n", len);
-		fprintf(stderr, "TTY: terms: ");
+		DEBUG_TTY("set terminators... %d bytes\n", len);
+		DEBUG_TTY("terms: ");
 		for (i=0; i<len; i++)
-			fprintf(stderr, "%02x ", buf[i]);
-		fprintf(stderr, "\n");
+			DEBUGLOG(DEBUG_FLAG_TTY, "%02x ", buf[i]);
+		DEBUGLOG(DEBUG_FLAG_TTY, "\n");
 	}
 	memset(terminators, 0, sizeof(terminators));
 	memcpy(terminators, buf, len);
@@ -194,10 +333,10 @@ void tty_set_terminators(char *buf, int len)
 void tty_start_read(char *prompt, int len, int promptlen)
 {
 	int i;
-	if (debug & 4)
-		fprintf(stderr, "TTY: start_read promptlen = %d, maxlen=%d\n",
-			promptlen, len);
-	if (promptlen) write(termfd, prompt, promptlen);
+	DEBUG_TTY("start_read promptlen = %d, maxlen=%d\n",
+		 promptlen, len);
+
+	if (promptlen) tty_write(prompt, promptlen);
 	if (len < 0) len = sizeof(input_buf);
 
 	/* Save the actual prompt in one buffer and the prefilled
@@ -205,7 +344,7 @@ void tty_start_read(char *prompt, int len, int promptlen)
 	memcpy(prompt_buf, prompt, promptlen);
 	prompt_len = promptlen;
 
-	/* Work out the position of the cursor after the prompt (which may contain CRLF chars ) */
+	/* Work out the position of the cursor after the prompt (which may contain CRLF chars) */
 	line_start_pos = 0;
 	for (i = 0; i<prompt_len; i++)
 		if (prompt_buf[i] >= ' ')
@@ -215,29 +354,23 @@ void tty_start_read(char *prompt, int len, int promptlen)
 	memcpy(input_buf, prompt+promptlen, len-promptlen);
 	input_len = len-promptlen;
 	input_pos = input_len;
-	write(termfd, input_buf, input_len);
-
-	max_read_len = len;
+	tty_write(input_buf, input_len);
+	reading = 1;
 
 	/* Now add in any typeahead */
 	if (rahead_len)
 	{
 		int copylen = rahead_len;
 
-		if (debug & 4)
-			fprintf(stderr, "TTY: readahead = %d bytes\n", rahead_len);
+		DEBUG_TTY("readahead = %d bytes\n", rahead_len);
 
 		/* Don't overflow the input buffer */
 		if (input_len + copylen > sizeof(input_buf))
 			copylen = sizeof(input_buf)-input_len;
-
-		memcpy(input_buf+input_len, rahead_buf, copylen);
-		tty_write(rahead_buf, copylen);
-		input_len += copylen;
-		input_pos += copylen;
-		rahead_len = 0;
+		rahead_len -= copylen;
+		tty_process_terminal(rahead_buf, copylen);
+		DEBUG_TTY("readahead now = %d bytes\n", rahead_len);
 	}
-	reading = 1;
 }
 
 void tty_set_timeout(unsigned short to)
@@ -253,6 +386,7 @@ void tty_clear_typeahead()
 
 void tty_set_maxlen(unsigned short len)
 {
+	DEBUG_TTY("max_read_len now = %d \n", len);
 	max_read_len = len;
 }
 
@@ -265,6 +399,8 @@ int tty_setup(char *name, int setup)
 	if (setup)
 	{
 		termfd = open(name, O_RDWR);
+		if (termfd == -1)
+			return -1;
 
 		tcgetattr(termfd, &old_term);
 		new_term = old_term;
@@ -297,9 +433,8 @@ static short is_terminator(char c)
 	aux = c % 8;
 	msk = (1 << aux);
 
-	if (debug & 4)
-		fprintf(stderr, "TTY: is_terminator: %d: (byte=%x, msk=%x) %s\n", c, terminators[termind],msk,
-			(terminators[termind] & msk)?"Yes":"No");
+	DEBUG_TTY("is_terminator: %d: (byte=%x, msk=%x) %s\n", c, terminators[termind],msk,
+		 (terminators[termind] & msk)?"Yes":"No");
 
 	if (terminators[termind] & msk)
 	{
@@ -311,7 +446,7 @@ static short is_terminator(char c)
 /* Erase to end of line */
 static void erase_eol(void)
 {
-	write(termfd, "\033[0K", 4);
+	tty_write("\033[0K", 4);
 }
 
 /* Move to column "hpos", where hpos starts at 0 */
@@ -320,7 +455,7 @@ static void move_cursor_abs(int hpos)
 	char buf[32];
 
 	sprintf(buf, "\r\033[%dC", hpos);
-	write(termfd, buf, strlen(buf));
+	tty_write(buf, strlen(buf));
 }
 
 
@@ -336,7 +471,8 @@ static void redraw_input_line(void)
 		return;
 	move_cursor_abs(line_start_pos);
 	erase_eol();
-	write(termfd, input_buf, input_len);
+	tty_write(input_buf, input_len);
+
 	move_cursor_abs(line_start_pos + input_pos);
 }
 
@@ -358,6 +494,11 @@ int tty_process_terminal(char *buf, int len)
 		if (buf[i] == '\n')
 			buf[i] = '\r';
 
+		/* I don't really understand this, but it's needed to make ^C work
+		   as ^Y at the VMS command-line */
+		if (buf[i] == CTRL_C && (!char_attr[(int)buf[i]] & 3))
+			buf[i] = CTRL_Y;
+
 		/* Check for OOB */
 		if (char_attr[(int)buf[i]] & 3)
 		{
@@ -371,6 +512,17 @@ int tty_process_terminal(char *buf, int len)
 			continue;
 		}
 
+		/* Read not active, store in the read ahead buffer */
+		if (!reading)
+		{
+			if (rahead_len == 0)
+				rahead_change(1); /* tell CTERM */
+
+			if (rahead_len < sizeof(rahead_buf))
+				rahead_buf[rahead_len++] = buf[i];
+			continue;
+		}
+
 		/* Is it ESCAPE ? */
 		if (buf[i] == ESC && esc_len == 0 && interpret_escape)
 		{
@@ -381,13 +533,12 @@ int tty_process_terminal(char *buf, int len)
 		/* Terminators */
 		if (!esc_len && is_terminator(buf[i]))
 		{
-			if (echo_terminator)
+			if (echo && echo_terminator)
 			{
 				tty_write(&buf[i], 1);
 			}
 			input_buf[input_len++] = buf[i];
-			send_input_buffer(SEND_FLAG_VALID_ESCAPE);
-			reading = 0;
+			send_input_buffer(SEND_FLAG_TERMINATOR);
 			continue;
 		}
 
@@ -395,7 +546,7 @@ int tty_process_terminal(char *buf, int len)
 		if (esc_len)
 		{
 			esc_buf[esc_len++] = buf[i];
-			if (isalpha(buf[i]) || buf[i] == '~')
+			if (!parse_escape(buf[i], &esc_state))
 			{
 				int esc_done = 0;
 
@@ -466,7 +617,7 @@ int tty_process_terminal(char *buf, int len)
 				input_buf[input_len++] = buf[i];
 				send_input_buffer(SEND_FLAG_TERMINATOR);
 				input_len = input_pos = 0;
-				reading = 0;
+				break;
 			case CTRL_O:
 				discard = ~discard;
 				break;
@@ -475,7 +626,7 @@ int tty_process_terminal(char *buf, int len)
 				{
 					input_pos--;
 					input_len--;
-					if (echo) write(termfd, "\033[D \033[D", 7);
+					if (echo) tty_write("\033[D \033[D", 7);
 					if (input_pos != input_len)
 					{
 						memmove(input_buf+input_pos, input_buf+input_pos+1, input_len-input_pos);
@@ -487,18 +638,7 @@ int tty_process_terminal(char *buf, int len)
 			continue;
 		}
 
-		/* Read not active, store in the read ahead buffer */
-		if (!reading)
-		{
-			if (rahead_len == 0)
-				rahead_change(1); /* tell CTERM */
-
-			if (rahead_len < sizeof(rahead_buf))
-				rahead_buf[rahead_len++] = buf[i];
-			continue;
-		}
-
-		if (convert_uppercase)
+		if (convert_uppercase && isalpha(buf[i]))
 			buf[i] &= 0x5F;
 
 		/* echo if req'd, insert will redraw the whole line */
@@ -520,6 +660,7 @@ int tty_process_terminal(char *buf, int len)
 
 		if (input_len >= max_read_len)
 		{
+			DEBUG_TTY("sending buffer, input_len=%d, max_read_len=%d\n", input_len, max_read_len);
 			send_input_buffer(SEND_FLAG_BUFFER_FULL);
 		}
 	}
