@@ -11,6 +11,8 @@
  *
  */
 
+#include <sys/types.h>
+#include <linux/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
